package com.book.server.service.impl;

import com.book.dao.utils.DateUtils;
import com.book.server.common.entity.Result;
import com.book.server.common.util.JsonUtils;
import com.book.server.config.SunTypeEnum;
import com.book.server.service.SunDataService;
import com.book.server.utils.DataUtil;
import com.book.server.utils.HttpTool;
import com.book.server.utils.HttpUtils;
import com.book.server.vo.*;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.stereotype.Service;

import java.nio.charset.StandardCharsets;
import java.util.*;

/**
 * created in 2021/9/28
 * Project: book-store
 *
 * @author win7
 */
@Service
@Slf4j
public class SunDataServiceImpl implements SunDataService {
    private static final String CLIENT_ID = "10005818";
    private static final String TOKEN = "PKp2HD39ze3CdzizqH";
    private static final String VIP_ID = "36637";
    private static final String URL = "https://data.yifengaf.cn";


    @Override
    public Result requestData(SunRequest request, SunTypeEnum sunTypeEnum) {
        try {

            String url = URL + sunTypeEnum.getPath();
            request.setClient_id(CLIENT_ID);
//            request.setToken(TOKEN);
            request.setNonce(UUID.randomUUID().toString().replace("-", ""));
            request.setTimestamp(System.currentTimeMillis() + "");
            request.setVip_id(VIP_ID);
            String signaure = DigestUtils.sha1Hex(TOKEN + request.getTimestamp() + CLIENT_ID + request.getNonce());
            request.setSignaure(signaure);

            String jsonStr = JsonUtils.toJsonStr(request);
            Map<String, Object> map = JsonUtils.getMap(jsonStr);
            String s = DataUtil.map2form(map);

            String result = HttpUtils.sendPost(url, s.getBytes(StandardCharsets.UTF_8));
            log.info("请求" + sunTypeEnum.getName() + ":" + JsonUtils.toJsonStr(request));
            log.info("响应" + sunTypeEnum.getName() + ":" + JsonUtils.toJsonStr(result));
            SunResult object = JsonUtils.getObject(result, SunResult.class);
            if (object != null && 0 == object.getError_code()) {
                return Result.success();
            } else {
                return object == null ? Result.failure() : Result.failure(object.getError_code(), object.getError_msg());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("sun 获取数据异常:" + e.getMessage());
        }
        return Result.failure();
    }

    @Override
    public void pullData(SunNotifyRequest request) {
        try {


            String type = request.getType();
            String path = request.getPath();
            Integer taskId = request.getTaskId();
            String url = URL + path;
            List<String> list = HttpTool.sendGet(url);
            if (list == null || list.isEmpty()) {
                log.error("读取sun文件为空");
                return;
            }
            SunTypeEnum sunTypeEnum = SunTypeEnum.valueOf(type);

            List result = new ArrayList();
            StringBuilder sb = new StringBuilder();
            for (int i = 0; i < list.size(); i++) {
                String line = list.get(i);
                switch (sunTypeEnum) {
                    case user:

                    case user_active:
                        SunTypeUser sunTypeUser = JsonUtils.getObject(line, SunTypeUser.class);
                        result.add(sunTypeUser);
                        break;
                    case user_collect:

                    case user_day_collect:
                        SunTypeUserDayCollect sunTypeUserDayCollect = JsonUtils.getObject(line, SunTypeUserDayCollect.class);
                        result.add(sunTypeUserDayCollect);
                        break;
                    case channel:
                        SunTypeChannel sunTypeChannel = JsonUtils.getObject(line, SunTypeChannel.class);
                        result.add(sunTypeChannel);
                        break;
                    case order:
                        SunTypeOrder sunTypeOrder = JsonUtils.getObject(line, SunTypeOrder.class);
                        result.add(sunTypeOrder);
                        break;
                    case custom_url:
                        SunTypeCustomUrl sunTypeCustomUrl = JsonUtils.getObject(line, SunTypeCustomUrl.class);
                        result.add(sunTypeCustomUrl);
                        break;
                    case custom_url_day_collect:
                        SunTypeCustomUrlCollect sunTypeCustomUrlCollect = JsonUtils.getObject(line, SunTypeCustomUrlCollect.class);
                        result.add(sunTypeCustomUrlCollect);
                        break;
                    case referral:
                        SunTypeReferral sunTypeReferral = JsonUtils.getObject(line, SunTypeReferral.class);
                        result.add(sunTypeReferral);
                        break;
                    case referral_day_collect:
                        SunTypeReferralCollect sunTypeReferralCollect = JsonUtils.getObject(line, SunTypeReferralCollect.class);
                        result.add(sunTypeReferralCollect);
                        break;
                    case activity:
                        SunTypeActivity sunTypeActivity = JsonUtils.getObject(line, SunTypeActivity.class);
                        result.add(sunTypeActivity);
                        break;
                    case activity_collect:

                    case activity_day_collect:
                        SunTypeActivityCollect sunTypeActivityCollect = JsonUtils.getObject(line, SunTypeActivityCollect.class);
                        result.add(sunTypeActivityCollect);
                        break;
                    default:
                        break;
                }

                Map map = new Gson().fromJson(line, Map.class);

                Set<String> set = map.keySet();
                if (i == 0) {
                    for (String key : set) {

                        sb.append(key + ",");
                    }

                    sb.append("\n");
                }
                for (String key : set) {

                    sb.append(map.get(key) + ",");
                }
                sb.append("\n");

            }
//        System.out.println(sb.toString());
            log.info(sb.toString());
        } catch (Exception e) {
            e.printStackTrace();
            log.error("sun解析数据出错:", e.getMessage());
        }
    }
}